@@ -4,23 +4,23 @@ import sys
 import os.path
 import re
 from pyfaidx import Fasta, wrap_sequence, FetchError, ucsc_split, bed_split
+from collections import defaultdict
 
 keepcharacters = (' ', '.', '_')
 
-
 def write_sequence(args):
     _, ext = os.path.splitext(args.fasta)
     if ext:
         ext = ext[1:]  # remove the dot from extension
     filt_function = re.compile(args.regex).search
-    fasta = Fasta(args.fasta, default_seq=args.default_seq, strict_bounds=not args.lazy, split_char=args.delimiter, filt_function=filt_function, rebuild=not args.no_rebuild)
+    fasta = Fasta(args.fasta, default_seq=args.default_seq, key_function=eval(args.header_function), strict_bounds=not args.lazy, split_char=args.delimiter, filt_function=filt_function, read_long_names=args.long_names, rebuild=not args.no_rebuild)
 
     regions_to_fetch, split_function = split_regions(args)
     if not regions_to_fetch:
         regions_to_fetch = fasta.keys()
     if args.invert_match:
         sequences_to_exclude = set([split_function(region)[0] for region in regions_to_fetch])
-        fasta = Fasta(args.fasta, default_seq=args.default_seq, strict_bounds=not args.lazy, split_char=args.delimiter, rebuild=not args.no_rebuild)
+        fasta = Fasta(args.fasta, default_seq=args.default_seq, key_function=eval(args.header_function), strict_bounds=not args.lazy, split_char=args.delimiter, rebuild=not args.no_rebuild)
         regions_to_fetch = (key for key in fasta.keys() if key not in sequences_to_exclude)
         split_function = ucsc_split
 
@@ -45,14 +45,14 @@ def write_sequence(args):
         try:
             if args.transform:
                 if not header and args.transform == 'nucleotide':
-                    outfile.write("name\tstart\tend\tA\tT\tC\tG\tN\n")
+                    outfile.write("name\tstart\tend\tA\tT\tC\tG\tN\tothers\n")
                     header = True
                 outfile.write(transform_sequence(args, fasta, name, start, end))
             else:
                 for line in fetch_sequence(args, fasta, name, start, end):
                     outfile.write(line)
         except FetchError as e:
-            raise FetchError(e.msg.rstrip() + "Try setting --lazy.\n")
+            raise FetchError(str(e) + " Try setting --lazy.\n")
         if args.split_files:
             outfile.close()
     fasta.__exit__()
@@ -61,7 +61,12 @@ def write_sequence(args):
 def fetch_sequence(args, fasta, name, start=None, end=None):
     try:
         line_len = fasta.faidx.index[name].lenc
-        sequence = fasta[name][start:end]
+        if args.auto_strand and start > end and start is not None and end is not None:
+            # flip (0, 1] coordinates
+            sequence = fasta[name][end - 1:start + 1]
+            sequence = sequence.reverse.complement
+        else:
+            sequence = fasta[name][start:end]
     except KeyError:
         sys.stderr.write("warning: {name} not found in file\n".format(**locals()))
         return
@@ -69,13 +74,13 @@ def fetch_sequence(args, fasta, name, start=None, end=None):
         sequence = sequence.complement
     if args.reverse:
         sequence = sequence.reverse
+    if args.no_output:
+        return
     if args.no_names:
         pass
-    elif args.full_names:
-        yield ''.join(['>', fasta[name].long_name, '\n'])
     else:
-        if start or end:
-            yield ''.join(['>', sequence.longname, '\n'])
+        if (start or end) and not args.no_coords:
+            yield ''.join(['>', sequence.fancy_name, '\n'])
         else:
             yield ''.join(['>', sequence.name, '\n'])
     for line in wrap_sequence(line_len, sequence.seq):
@@ -114,14 +119,23 @@ def split_regions(args):
 def transform_sequence(args, fasta, name, start=None, end=None):
     line_len = fasta.faidx.index[name].lenc
     s = fasta[name][start:end]
+    if args.no_output:
+        return
     if args.transform == 'bed':
-        return '{name}\t{start}\t{end}\n'.format(name=s.name, start=s.start, end=s.end)
+        return '{name}\t{start}\t{end}\n'.format(name=s.name, start=s.start - 1 , end=s.end)
     elif args.transform == 'chromsizes':
         return '{name}\t{length}\n'.format(name=s.name, length=len(s))
     elif args.transform == 'nucleotide':
-        nucs = Counter(dict([('A', 0), ('T', 0), ('C', 0), ('G', 0), ('N', 0)]))
-        nucs.update(str(s).upper())
-        return '{name}\t{start}\t{end}\t{A}\t{T}\t{C}\t{G}\t{N}\n'.format(name=s.name, start=s.start, end=s.end, **nucs)
+        ss = str(s).upper()
+        nucs = defaultdict(int)
+        nucs.update([(c, str(ss).count(c)) for c in set(str(ss))])
+        A = nucs.pop('A', 0)
+        T = nucs.pop('T', 0)
+        C = nucs.pop('C', 0)
+        G = nucs.pop('G', 0)
+        N = nucs.pop('N', 0)
+        others = '|'.join([':'.join((k, v)) for k, v in nucs.items()])
+        return '{sname}\t{sstart}\t{send}\t{A}\t{T}\t{C}\t{G}\t{N}\t{others}\n'.format(sname=s.name, sstart=s.start, send=s.end, **locals())
     elif args.transform == 'transposed':
         return '{name}\t{start}\t{end}\t{seq}\n'.format(name=s.name, start=s.start, end=s.end, seq=str(s))
 
@@ -133,25 +147,33 @@ def main(ext_args=None):
                                      epilog="Please cite: Shirley MD, Ma Z, Pedersen BS, Wheelan SJ. (2015) Efficient \"pythonic\" access to FASTA files using pyfaidx. PeerJ PrePrints 3:e1196 https://dx.doi.org/10.7287/peerj.preprints.970v1")
     parser.add_argument('fasta', type=str, help='FASTA file')
     parser.add_argument('regions', type=str, nargs='*', help="space separated regions of sequence to fetch e.g. chr1:1-1000")
-    parser.add_argument('-b', '--bed', type=argparse.FileType('r'), help="bed file of regions")
-    parser.add_argument('-o', '--out', type=argparse.FileType('w'), help="output file name (default: stdout)")
-    parser.add_argument('-i', '--transform', type=str, choices=('bed', 'chromsizes', 'nucleotide', 'transposed'), help="transform the requested regions into another format. default: %(default)s")
-    parser.add_argument('-c', '--complement', action="store_true", default=False, help="complement the sequence. default: %(default)s")
-    parser.add_argument('-r', '--reverse', action="store_true", default=False, help="reverse the sequence. default: %(default)s")
-    parser.add_argument('-a', '--size-range', type=parse_size_range, default=None, help='selected sequences are in the size range [low, high]. example: 1,1000 default: %(default)s')
-    names = parser.add_mutually_exclusive_group()
+    _input = parser.add_argument_group('input options')
+    output = parser.add_argument_group('output options')
+    header = parser.add_argument_group('header options')
+    _input.add_argument('-b', '--bed', type=argparse.FileType('r'), help="bed file of regions")
+    output.add_argument('-o', '--out', type=argparse.FileType('w'), help="output file name (default: stdout)")
+    output.add_argument('-i', '--transform', type=str, choices=('bed', 'chromsizes', 'nucleotide', 'transposed'), help="transform the requested regions into another format. default: %(default)s")
+    output.add_argument('-c', '--complement', action="store_true", default=False, help="complement the sequence. default: %(default)s")
+    output.add_argument('-r', '--reverse', action="store_true", default=False, help="reverse the sequence. default: %(default)s")
+    output.add_argument('-y', '--auto-strand', action="store_true", default=False, help="reverse complement the sequence when start > end coordinate. default: %(default)s")
+    output.add_argument('-a', '--size-range', type=parse_size_range, default=None, help='selected sequences are in the size range [low, high]. example: 1,1000 default: %(default)s')
+    names = header.add_mutually_exclusive_group()
     names.add_argument('-n', '--no-names', action="store_true", default=False, help="omit sequence names from output. default: %(default)s")
-    names.add_argument('-f', '--full-names', action="store_true", default=False, help="output full names including description. default: %(default)s")
-    parser.add_argument('-x', '--split-files', action="store_true", default=False, help="write each region to a separate file (names are derived from regions)")
-    parser.add_argument('-l', '--lazy', action="store_true", default=False, help="fill in --default-seq for missing ranges. default: %(default)s")
-    parser.add_argument('-s', '--default-seq', type=check_seq_length, default='N', help='default base for missing positions and masking. default: %(default)s')
-    parser.add_argument('-d', '--delimiter', type=str, default=None, help='delimiter for splitting names to multiple values (duplicate names will be discarded). default: %(default)s')
-    matcher = parser.add_mutually_exclusive_group()
+    names.add_argument('-f', '--long-names', action="store_true", default=False, help="output full (long) names from the input fasta headers. default: headers are truncated after the first whitespace")
+    header.add_argument('-t', '--no-coords', action="store_true", default=False, help="omit coordinates (e.g. chr:start-end) from output headers. default: %(default)s")
+    output.add_argument('-x', '--split-files', action="store_true", default=False, help="write each region to a separate file (names are derived from regions)")
+    output.add_argument('-l', '--lazy', action="store_true", default=False, help="fill in --default-seq for missing ranges. default: %(default)s")
+    output.add_argument('-s', '--default-seq', type=check_seq_length, default='N', help='default base for missing positions and masking. default: %(default)s')
+    header.add_argument('-d', '--delimiter', type=str, default=None, help='delimiter for splitting names to multiple values (duplicate names will be discarded). default: %(default)s')
+    header.add_argument('-e', '--header-function', type=str, default='lambda x: x.split()[0]', help='python function to modify header lines e.g: "lambda x: x.split("|")[0]". default: %(default)s')
+    header.add_argument('-u', '--duplicates-action', type=str, default="stop", choices=("stop", "first", "last", "longest", "shortest"), help='entry to take when duplicate sequence names are encountered. default: %(default)s')
+    matcher = header.add_mutually_exclusive_group()
     matcher.add_argument('-g', '--regex', type=str, default='.*', help='selected sequences are those matching regular expression. default: %(default)s')
     matcher.add_argument('-v', '--invert-match', action="store_true", default=False, help="selected sequences are those not matching 'regions' argument. default: %(default)s")
-    masking = parser.add_mutually_exclusive_group()
+    masking = output.add_mutually_exclusive_group()
     masking.add_argument('-m', '--mask-with-default-seq', action="store_true", default=False, help="mask the FASTA file using --default-seq default: %(default)s")
     masking.add_argument('-M', '--mask-by-case', action="store_true", default=False, help="mask the FASTA file by changing to lowercase. default: %(default)s")
+    output.add_argument('--no-output', action="store_true", default=False, help="do not output any sequence. default: %(default)s")
     parser.add_argument('--no-rebuild', action="store_true", default=False, help="do not rebuild the .fai index even if it is out of date. default: %(default)s")
     parser.add_argument('--version', action="version", version=__version__, help="print pyfaidx version number")
     # print help usage if no arguments are supplied
@@ -162,12 +184,19 @@ def main(ext_args=None):
         args = parser.parse_args(ext_args)
     else:
         args = parser.parse_args()
+        
+    if args.auto_strand:
+        if args.complement:
+            sys.stderr.write("--auto-strand and --complement are both set. Are you sure this is what you want?\n")
+        if args.reverse:
+            sys.stderr.write("--auto-strand and --reverse are both set. Are you sure this is what you want?\n")
 
     if args.mask_with_default_seq or args.mask_by_case:
         mask_sequence(args)
     else:
         write_sequence(args)
 
+    
 
 def check_seq_length(value):
     if len(value) != 1:
